#!/usr/bin/env ruby
#
#############################################################
#
# file    : rbalbum.rb
# author  : mathias gumz <akira at fluxbox dot org>
# copyr   : copyright (c) 2005 by m. gumz
# license : see LICENSE
#
# start   : So 20 Feb 2005 18:45:30 CET
#
#############################################################
#
# about :
#
#   creates an photo-album from a list of images with an 
#   index-file + subpages for each image
#
#############################################################
#
#
#

#=begin RC
#
# rbalbum config - file
# autocreated by rbalbum --dump-config
RC = {

    # flags
    :flags => {
        :verbose => false,
        :use_thumbnails => true,
        :create_index => true,
        :create_pages => true,
        :create_thumbnails => true,
        :scale_images => false
    },

    # options for the image-index
    :index_opts => {
        :title => 'rbalbum',
        :heading => 'rbalbum', 
        :csslink => 'rbalbum.css',
        :cssid => 'rbalbum',
        :indexfile => 'index.html',
        :entries_per_row => 5,
        :container_tags => { :open => '<table>', :close => '</table>' },
        :row_tags => { :open => '<tr>', :close => '</tr>' },
        :col_tags => { :open => '<td>', :close => '</td>' }
    },
    
    # options for images
    :image_opts => {
        :pattern => '*.{jpg,JPG,png,PNG}',
    },

    # options for scaling
    :scale_opts => {
        :directory => 'invalid directory',
        :max_width => 500,
        :max_height => 500,
        :max_qual => 95,
        :min_qual => 50,
        :max_size => 50000
    },
    
    # options for thumbnails
    :thumb_opts => {
        :prefix => 'TN_',
        :width => 100,
        :height => 80
    },

    # options for imagepages
    :imagepage_opts => {
        :title => 'rbalbum',
        :csslink => 'rbalbum.css',
        :cssid => 'rbalbum',
        :cssnavid => 'rbalbum_nav',
        
        # labeling of the nav-elements on each imagepage
        :prev => 'prev',
        :next => 'next',
        :toc =>  'toc'

    },

    # html - atoms
    :atoms => { }
}

RC[:atoms][:header] = <<EOF
<html>
<head>
<title>@TITLE@</title>
<meta name="generator" content="rbalbum - http://darkshed.net/projects">
<link rel="stylesheet" type="text/css" href="@CSSLINK@">
</head>
<body>
EOF

RC[:atoms][:footer] = <<EOF
</body>
</html>
EOF

RC[:atoms][:index_opening] = <<EOF
<div id="@CSSID@">
<center>
<h1>@HEADING@</h1>
EOF

RC[:atoms][:index_closing] = <<EOF
</center>
</div>
EOF

RC[:atoms][:imagepage_body] = <<EOF
<div id='@CSSID@'>
<center>
<h1>@IMAGE@ - @INDEX@/@COUNT@</h1>
<img src='@IMAGE@' />
    <div id='@CSSNAV@'>
        <center>
            <table>
                <tr>
                  <td><div class='prev'><a href='@LINKPREV@'>@TEXTPREV@</a></div></td>
                  <td><div class='toc'><a href='@LINKTOC@'>@TEXTTOC@</a></div></td>
                  <td><div class='next'><a href='@LINKNEXT@'>@TEXTNEXT@</a></div></td>
                </tr>
            </table>
        </center>
    </div>
</center>
</div>
EOF
#=end RC

#############################################################
# 
class String
def to_bool()
    if self =~ /true|TRUE|1/ then 
        return true
    else 
        return false
    end
end
end




def displayHelp()
    puts <<EOF
ABOUT
    
    #{$0} creates a html-album from a collection of pictures. it scans the
    current directory for 'thumbnail'-images and associates them with the
    'real' images, creates then an index-file from it and subpages for every
    image, all linked together. furthermore, it can collect a bunch of images,
    scale them down to fit a given geometry  and reduce the quality to obtain
    a given size .. so everything is just as big as it needs to be.

USAGE

    #{$0} [-h] [-c config] [-pitTv bool] [-C columns ] [-d directory]

OPTIONS

    -h     --help           
    
             display this help
            
    -v     --verbose <true/false>

             shows whats going on

    -c     --config <file>
    
            load contents of <file> and merges the content of the 
            USERRC - structure with the default settings.
            
    -d     --dump-config
    
             prints the config

    -p     --create-pages <true/false>

             for each image a new page is created, default is to 
             add a little navigationelement below the image and 
             display some info.
    
    -i     --create-index <true/false>

             the index-page is the list of all images, showing a 
             preview by using thumbnails and offering links to all 
             the images.
             
    -t     --use-thumbs <true/false>   

             use thumbnails if available or wanted. if not, pure 
             textlink to the image.
             
    -T     --create-thumbs <true/false>
    
             create thumbnails (using RMagick) for each image which 
             isnt a thumbnail already. you can specify a prefix to 
             use for the thumbnail-name, default is 'TN_'.
             
    -s     --scale-images <true/false>
    
             runs over all images and scales them to fit a given
             limit for the geometry. after that it decreases the
             quality of the images so they wont be bigger than
             a certain value. it takes the images from -d directory
             and copies them to the current directory before scaling
             them.
             
    -C     --columns <number>    
    
             how many columns for the index

    -D     --image-directory <directory>
            
             directory containing all the images which should be used
             with the --scale-images routine
    
AUTHOR

    mathias gumz <akira at fluxbox dot org>

EOF
end


######################
# parsing commandline

require 'getoptlong'
cmdline = GetoptLong.new
cmdline.set_options(
    [ "--help",            "-h",  GetoptLong::NO_ARGUMENT ],
    [ "--config",          "-c",  GetoptLong::REQUIRED_ARGUMENT ],
    [ "--columns",         "-C",  GetoptLong::REQUIRED_ARGUMENT ],
    [ "--create-pages",    "-p",  GetoptLong::REQUIRED_ARGUMENT ],
    [ "--create-index",    "-i",  GetoptLong::REQUIRED_ARGUMENT ],
    [ "--create-thumbs",   "-T",  GetoptLong::REQUIRED_ARGUMENT ],
    [ "--scale-images",    "-s",  GetoptLong::REQUIRED_ARGUMENT ],
    [ "--use-thumbs",      "-t",  GetoptLong::REQUIRED_ARGUMENT ],
    [ "--dump-config",     "-d",  GetoptLong::NO_ARGUMENT ],
    [ "--verbose",         "-v",  GetoptLong::REQUIRED_ARGUMENT ],
    [ "--image-directory", "-D",  GetoptLong::REQUIRED_ARGUMENT ]
)

cmdline.each do |opt, arg|
    case opt
    when /--config/
        config = arg.strip
        if File.exist?(config) then
            begin
                load(config)
            rescue
                puts "error while loading config [#{config}].exit."
                exit(1)
            end
            if USERRC then
                RC.merge!(USERRC)
            end
        end
    when /--create-pages/ then  RC[:flags][:create_pages] = arg.strip.to_bool
    when /--create-index/ then  RC[:flags][:create_index] = arg.strip.to_bool
    when /--create-thumbs/ then RC[:flags][:create_thumbnails] = arg.strip.to_bool
    when /--use-thumbs/ then    RC[:flags][:use_thumbnails] = arg.strip.to_bool
    when /--scale-images/ then  RC[:flags][:scale_images] = arg.strip.to_bool
    when /--verbose/ then       RC[:flags][:verbose] = arg.strip.to_bool
    when /--image-directory/ then RC[:scale_opts][:directory] = arg.strip
    when /--columns/ 
        RC[:index_opts][:entries_per_row] = arg.strip.to_i if arg.strip.to_i > 0
    when /--help/
        displayHelp
        exit(0)
    when /--dump-config/
        File.open($0, 'r') do |file|
            dump = false
            file.each_line do |line|
                break if line =~ /^#=end RC$/
                ( dump = true ; next ) if line =~ /^#=begin RC$/ 
                puts line.gsub(/RC/,'USERRC') if dump
            end
        end
        exit(0)
    else 
        displayHelp
        exit(1)
    end
end





flags = RC[:flags]
t_opts = RC[:thumb_opts]
i_opts = RC[:image_opts]
indent = $0.gsub(/./, ' ')
verbose = RC[:flags][:verbose]

if flags[:scale_images] then
    
    begin 
        require 'RMagick'
        include Magick
    rescue
        puts "error, no 'RMagick' installed. exit."
        exit(1)
    end

    puts " == scaling images ==\n\n" if verbose

    opts = RC[:scale_opts]
    
    if File.exists?(opts[:directory]) then
        Dir.glob(opts[:directory] + i_opts[:pattern]) do |i|
            img = Magick::Image.read(i).first

            print "   #{i} #{img.columns}x#{img.rows} .. " if verbose
            
            if img.columns > opts[:max_width] and img.columns > img.rows then
                img.change_geometry!(opts[:max_width].to_s + 'x') do |col,row,image|
                    image.resize!(col, row)
                end
            elsif img.rows > opts[:max_height] and img.rows > img.columns then
                img.change_geometry!('x' + opts[:max_height].to_s) do |col,row,image| 
                    image.resize!(col, row)
                end
            end
                
            print "#{img.columns}x#{img.rows} " if verbose
            
            pattern = Regexp.new(opts[:directory] + '(.*)')
            name = pattern.match(i)[1]
            tmpname = 'tmp_' + name

            img.write(tmpname)
            if File.size(tmpname) > opts[:max_size] then
            
                (-opts[:max_qual]..-opts[:min_qual]).step(5) do |q|

                    print "#{-q} " if verbose
                    
                    imgnew = img.copy
                    imgnew.write(tmpname) { self.quality = -q }
                    if File.size(tmpname) <= opts[:max_size] then 
                        break
                    end
                end
            end
            File.rename(tmpname, name)
            puts " .. #{name} [ok]" if verbose
        end
    else
        puts "error, no image-directory found."
        exit(1)
    end
end

######################################################
# create the thumbnails itself
if flags[:create_thumbnails] then

    begin
        require 'RMagick'
        include Magick
    rescue
        puts "error, no 'RMagick' installed. exit."
        exit(1)
    end
 
    puts " == creating thumbnails ==\n\n" if verbose
    
    regexp = Regexp.new(t_opts[:prefix] + '(.*)')
    Dir.glob(i_opts[:pattern]) do |i|
        if not i =~ regexp then
            print "  #{i} .. " if verbose
            img = Magick::Image.read(i).first
            if img.columns < img.rows then
                img.scale!(t_opts[:height].to_f / img.rows)
            else
                img.scale!(t_opts[:width].to_f / img.columns)
            end
            name = t_opts[:prefix] + i
            img.write(name)
            puts "#{name} [ok]" if verbose
        end
    end

    puts "\n\n" if verbose
end

######################################################
# collect image-files, create image-thumbnail-mapping

images = [ ]

puts " == collecting images ==\n\n" if verbose
if flags[:use_thumbnails] then

    regexp = Regexp.new(t_opts[:prefix] + '(.*)')
    Dir.glob(t_opts[:prefix] + i_opts[:pattern]) do |thumb|
        img = regexp.match(thumb)[1]
        images << { :img => img, :thumb => thumb } if File.exists?(img)
    end
else
    Dir.glob(i_opts[:pattern]) do |img|
        images << { :img => img, :thumb => nil }
    end
end

if images.size < 1 then
    puts 'error, no images found. exit.'
    exit(1)
end

puts "    found #{images.size} [ok]\n\n" if verbose

######################################################
# sorting

images.sort! { |a,b| a[:img] <=> b[:img] }

#############################################################
# creating the image-index
if flags[:create_index] then

    puts " == creating index ==\n\n" if verbose
    
    opts = RC[:index_opts]
    atoms = RC[:atoms]

    header = String.new(atoms[:header])
        header.gsub!(/@TITLE@/, opts[:title])
        header.gsub!(/@CSSLINK@/, opts[:csslink])
    openening = String.new(atoms[:index_opening])
        openening.gsub!(/@HEADING@/, opts[:heading])
        openening.gsub!(/@CSSID@/, opts[:cssid])
    closing = String.new(atoms[:index_closing])
    footer = String.new(atoms[:footer])

    File.open(opts[:indexfile], 'w') do |index|

        index.puts header
        index.puts openening
        index.puts opts[:container_tags][:open]

        tmp = Array.new(images)
        (opts[:entries_per_row] - images.size % opts[:entries_per_row]).times { tmp << nil }
        (tmp.size / opts[:entries_per_row]).times do |i|
            
            index.puts opts[:row_tags][:open] + "<!-- row #{i} -->"
            tmp[(i * opts[:entries_per_row])...((i + 1)* opts[:entries_per_row])].each do |j|
                if j then

                    link = flags[:create_pages] ?
                        /(.*)\..*$/.match(j[:img])[1] + '.html' : j[:img]

                    index.puts opts[:col_tags][:open] + 
                          "<a href='"+ link + "'"  + "'>" +
                          ( j[:thumb] ? "<img src='" + j[:thumb] + "' alt='" +
                          j[:img] + "'/>" : "") + 
                          "<br/>" + j[:img] + "</a>" +
                          opts[:col_tags][:close]
                else
                    index.puts opts[:col_tags].values.join.to_s
                end
            end
            index.puts opts[:row_tags][:close]
        end
        index.puts opts[:container_tags][:close]
        index.puts closing
        index.puts footer
    end

    puts "    #{opts[:indexfile]} [ok]\n\n" if verbose
end

#############################################################
# creating the per-image-pages
if flags[:create_pages] then

    puts " == creating imagepages ==\n\n" if verbose

    opts = RC[:imagepage_opts]
    atoms = RC[:atoms]

    header = String.new(atoms[:header])
        header.gsub!(/@TITLE@/, opts[:title])
        header.gsub!(/@CSSLINK@/, opts[:csslink])
    footer = String.new(atoms[:footer])

    images.each_index do |index|
        
        name = /(.*)\..*$/.match(images[index][:img])[1] + '.html'
        print "  #{index} .. " if verbose

        File.open(name, 'w') do |page|

            body = String.new(RC[:atoms][:imagepage_body])
            toc_page = flags[:create_index] ? RC[:index_opts][:indexfile] : ""
            next_page = /(.*)\..*$/.match(images[(index + 1) % images.size][:img])[1] + '.html'
            prev_page = /(.*)\..*$/.match(images[index - 1][:img])[1] + '.html'

            body.gsub!(/@IMAGE@/, images[index][:img])
            body.gsub!(/@THUMB@/, images[index][:thumb] ? images[index][:thumb] : "" )
            body.gsub!(/@CSSID@/, opts[:cssid])
            body.gsub!(/@CSSNAV@/, opts[:cssnavid])
            body.gsub!(/@LINKPREV@/, prev_page)
            body.gsub!(/@LINKTOC@/, toc_page)
            body.gsub!(/@LINKNEXT@/, next_page)
            body.gsub!(/@TEXTPREV@/, opts[:prev])
            body.gsub!(/@TEXTTOC@/, flags[:create_index] ? opts[:toc] : "")
            body.gsub!(/@TEXTNEXT@/, opts[:next])
            body.gsub!(/@INDEX@/, (index + 1).to_s)
            body.gsub!(/@COUNT@/, images.size.to_s)

            page.puts header
            page.puts body
            page.puts footer
        end
        puts "#{name} [ok]" if verbose
    end

    puts "\n" if verbose
end

puts " == done =================\n\n" if verbose

#
#
#
#############################################################
#
#
#############################################################

